;==========================================================================
;  Jobeaux Processor Display Driver Module
; 
;  Copyright (C) 2011
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Revision History:
;	Who		Date	Description
;	sgt		TBD		Initial Release
;==========================================================================

#include "jobeaux.inc"
	SUBTITLE "Display Driver Module"

	GLOBAL	InitDisplay, TendDisplay, DisplaySplash
	GLOBAL	msDelay
	GLOBAL	DisplaySetupScreen

	EXTERN	CheckWaterLevel
	EXTERN	DisplayTimer, DisplaySetTemp, DisplayCurTemp, DisplayLowTemp
	EXTERN	CountToTemp
	EXTERN	SetupScr1, SetupScr2, SetupScr3
	EXTERN	OKUpdate

;==========================================================================
; Shared variables for cursor position
;==========================================================================
	UDATA_SHR
Column			RES	1
DelayCounter	RES	1

	UDATA
LoopCounter		RES	1


DISPLAY	CODE

;==========================================================================
; Update the display
; Modifies: bank, page
;==========================================================================
TendDisplay:
		; Set Data Mode
		banksel	LCD_MODE_PORT		; Bank 0
		bsf		LCD_MODE_PORT, LCD_MODE_OUT

		; If water level is too low, display warning message
		pagesel	CheckWaterLevel
		call	CheckWaterLevel
		btfss	WREG, 0
		goto	DisplayWarning

		movf	OKUpdate, F			; Only update if the timer has ticked
		btfsc	STATUS, Z
		return

		clrf	OKUpdate

		; Line 1 = Desired Temp
		movlw	0					; 0 = Convert Desired temp
		pagesel	CountToTemp
		call	CountToTemp

		clrf	Column
setloop:
		movfw	Column
		incf	Column, F
		pagesel	DisplaySetTemp
		call	DisplaySetTemp		; Get next char for line 1
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	setloop

		; Line 2 = Current Temp
		clrf	Column
		movlw	1					; 1 = Convert Current temp
		pagesel	CountToTemp
		call	CountToTemp
		andlw	0xFF
		btfss	STATUS, Z
		goto	curloop				; Valid conversion

		; Current temperature is below display range
lowTemp:
		movfw	Column
		incf	Column, F
		pagesel	DisplayLowTemp
		call	DisplayLowTemp		; Get next char for line 2
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	lowTemp
		goto	dispTimer

curloop:
		movfw	Column
		incf	Column, F
		pagesel	DisplayCurTemp
		call	DisplayCurTemp		; Get next char for line 2
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	curloop

dispTimer:
		; Line 3 = Timer
		clrf	Column
timerloop:
		movfw	Column
		incf	Column, F
		pagesel	DisplayTimer
		call	DisplayTimer		; Get next char for line 3
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	timerloop

		return



;==========================================================================
; Display a warning message that the water level is too low
; Modifies: bank, page
;==========================================================================
DisplayWarning:
		clrf	Column
warn1:
		movfw	Column
		incf	Column, F
		pagesel	Warning1
		call	Warning1			; Get next char for line 1
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	warn1

		clrf	Column
warn2:
		movfw	Column
		incf	Column, F
		pagesel	Warning2
		call	Warning2			; Get next char for line 2
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	warn2

		clrf	Column
warn3:
		movfw	Column
		incf	Column, F
		pagesel	Warning3
		call	Warning3			; Get next char for line 3
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	warn3

		return


;==========================================================================
; Warning1: Line 1 of low water level warning
;==========================================================================
Warning1:
		brw
		retlw	'*'					; Column 0
		retlw	'*'					; Column 1
		retlw	'*'					; Column 2
		retlw	' '					; Column 3
		retlw	' '					; Column 4
		retlw	'C'					; Column 5
		retlw	'h'					; Column 6
		retlw	'e'					; Column 7
		retlw	'c'					; Column 8
		retlw	'k'					; Column 9
		retlw	' '					; Column 10
		retlw	' '					; Column 11
		retlw	' '					; Column 12
		retlw	'*'					; Column 13
		retlw	'*'					; Column 14
		retlw	'*'					; Column 15

;==========================================================================
; Warning2: Line 2 of low water level warning
;==========================================================================
Warning2:
		brw
		retlw	'*'					; Column 0
		retlw	'*'					; Column 1
		retlw	'*'					; Column 2
		retlw	' '					; Column 3
		retlw	' '					; Column 4
		retlw	'W'					; Column 5
		retlw	'a'					; Column 6
		retlw	't'					; Column 7
		retlw	'e'					; Column 8
		retlw	'r'					; Column 9
		retlw	' '					; Column 10
		retlw	' '					; Column 11
		retlw	' '					; Column 12
		retlw	'*'					; Column 13
		retlw	'*'					; Column 14
		retlw	'*'					; Column 15

;==========================================================================
; Warning3: Line 3 of low water level warning
;==========================================================================
Warning3:
		brw
		retlw	'*'					; Column 0
		retlw	'*'					; Column 1
		retlw	'*'					; Column 2
		retlw	' '					; Column 3
		retlw	' '					; Column 4
		retlw	'L'					; Column 5
		retlw	'e'					; Column 6
		retlw	'v'					; Column 7
		retlw	'e'					; Column 8
		retlw	'l'					; Column 9
		retlw	' '					; Column 10
		retlw	' '					; Column 11
		retlw	' '					; Column 12
		retlw	'*'					; Column 13
		retlw	'*'					; Column 14
		retlw	'*'					; Column 15


;==========================================================================
; Display a splash screen on the LCD for 5 seconds
; Modifies: bank, page
;==========================================================================
DisplaySplash:
		clrf	Column
spl1:
		movfw	Column
		incf	Column, F
		pagesel	Splash1
		call	Splash1				; Get next char for line 1
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	spl1

		clrf	Column
spl2:
		movfw	Column
		incf	Column, F
		pagesel	Splash2
		call	Splash2				; Get next char for line 2
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	spl2

		clrf	Column
spl3:
		movfw	Column
		incf	Column, F
		pagesel	Splash3
		call	Splash3				; Get next char for line 3
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	spl3

		; Wait 5 seconds (50 * 100 msec)
		movlw	50
		banksel	LoopCounter
		movwf	LoopCounter
splashDelay:
		movlw	100
		pagesel	msDelay
		call	msDelay
		decfsz	LoopCounter, F
		goto	splashDelay
		return


;==========================================================================
; Splash1: Line 1 of splash screen
;==========================================================================
Splash1:
		brw
		retlw	'J'					; Column 0
		retlw	'o'					; Column 1
		retlw	'b'					; Column 2
		retlw	'e'					; Column 3
		retlw	'a'					; Column 4
		retlw	'u'					; Column 5
		retlw	'x'					; Column 6
		retlw	' '					; Column 7
		retlw	' '					; Column 8
		retlw	' '					; Column 9
		retlw	' '					; Column 10
		retlw	' '					; Column 11
		retlw	' '					; Column 12
		retlw	' '					; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15

;==========================================================================
; Splash2: Line 2 of splash screen
;==========================================================================
Splash2:
		brw
		retlw	'P'					; Column 0
		retlw	'r'					; Column 1
		retlw	'o'					; Column 2
		retlw	'c'					; Column 3
		retlw	'e'					; Column 4
		retlw	's'					; Column 5
		retlw	's'					; Column 6
		retlw	'o'					; Column 7
		retlw	'r'					; Column 8
		retlw	' '					; Column 9
		retlw	' '					; Column 10
		retlw	' '					; Column 11
		retlw	' '					; Column 12
		retlw	' '					; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15

;==========================================================================
; Splash3: Line 3 of splash screen
;==========================================================================
Splash3:
		brw
		retlw	'R'					; Column 0
		retlw	'e'					; Column 1
		retlw	'v'					; Column 2
		retlw	' '					; Column 3
		retlw	'0'					; Column 4
		retlw	'.'					; Column 5
		retlw	'0'					; Column 6
		retlw	' '					; Column 7
		retlw	' '					; Column 8
		retlw	' '					; Column 9
		retlw	' '					; Column 10
		retlw	' '					; Column 11
		retlw	' '					; Column 12
		retlw	' '					; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15


;==========================================================================
; Display the setup screen to allow user to configure temp units
; Modifies: bank, page
;==========================================================================
DisplaySetupScreen:
		clrf	Column
setup1:
		movfw	Column
		incf	Column, F
		pagesel	SetupScr1
		call	SetupScr1			; Get next char for line 1
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	setup1

		clrf	Column
setup2:
		movfw	Column
		incf	Column, F
		pagesel	SetupScr2
		call	SetupScr2			; Get next char for line 2
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	setup2

		clrf	Column
setup3:
		movfw	Column
		incf	Column, F
		pagesel	SetupScr3
		call	SetupScr3			; Get next char for line 3
		pagesel	LCDOut
		call	LCDOut
		movfw	Column
		andlw	0x0F
		btfss	STATUS, Z
		goto	setup3

		return


;==========================================================================
; Output a byte (command or data) to the display
; Caller must set RA1 output to indicate cmd/data
; Byte to be sent is in W
; Returns with W unchanged, Bank 0 selected
;==========================================================================
LCDOut:
		; Set Chip Select
		banksel	LCD_ENABLE_PORT		; Bank 0
		bcf		LCD_ENABLE_PORT, LCD_ENABLE_OUT 

		; Send the char
		banksel	SSP1BUF				; Bank 4
		movwf	SSP1BUF

		; Wait for completion
		btfss	SSP1STAT, BF
		goto	$-1

		; Deassert Chip Select
		banksel	LCD_ENABLE_PORT		; Bank 0
		bsf		LCD_ENABLE_PORT, LCD_ENABLE_OUT 

		; Most commands take about 20 usec to complete
		; There's 10 usec overhead in the call and return
		; so let's waste the other 10 here just to be safe
		; For those that take longer (Clear Display, Home),
		; the caller must add appropriate delay
		goto	$+1					; 2 usec
		goto	$+1					; 2 usec
		goto	$+1					; 2 usec
		goto	$+1					; 2 usec
		goto	$+1					; 2 usec

		return


;==========================================================================
; Initialize the DOGM163 display
; Modifies: bank, page
;==========================================================================
InitDisplay:
		; Wait 40msec before attempting to talk to LCD
		movlw	40
		call	msDelay

		; Set Command Mode
		banksel	LCD_MODE_PORT		; Bank 0
		bcf		LCD_MODE_PORT, LCD_MODE_OUT

		pagesel	LCDOut
		movlw	B'00111001'			; Function Set
		call	LCDOut
		movlw	B'00011101'			; Bias Set
		call	LCDOut
		movlw	B'01010000'			; Power Control
		call	LCDOut
		movlw	B'01101100'			; Follower Control
		call	LCDOut
		movlw	B'01111100'			; Contrast Set
		call	LCDOut
		movlw	B'00111000'			; Function Set
		call	LCDOut
		movlw	B'00001100'			; Display ON, no cursor
		call	LCDOut
		movlw	B'00000001'			; Clear Display
		call	LCDOut
		movlw	1
		pagesel	msDelay
		call	msDelay
		pagesel	LCDOut
		movlw	B'00000110'			; Entry Mode Set
		call	LCDOut

		; Set Data Mode
		bsf		LCD_MODE_PORT, LCD_MODE_OUT

		return


;==========================================================================
; Delay the number of milliseconds in W
; (Actual: W*1000 + 4 usec)
; Assumes 4 MHz system clock
; W range 1-255
; Does not change selected register bank or memory page
;==========================================================================
msDelay:
		movwf	DelayCounter		; 1 usec

delayLoop:
		; Each time through this inner loop is 500 usec
		; (166 * 3 + 2)
		movlw	166					; 1 usec
		decfsz	WREG, W				; 1 usec (2 on skip)
		goto	$-1					; 2 usec

		; Do it again for 497 usec
		; (165 * 3 + 2)
		movlw	165					; 1 usec
		decfsz	WREG, W				; 1 usec (2 on skip)
		goto	$-1					; 2 usec

		decfsz	DelayCounter, F		; 1 usec (2 on skip)
		goto	delayLoop			; 2 usec

		return						; 2 usec

	end
