;==========================================================================
;  Jobeaux Processor EEPROM Module
; 
;  Copyright (C) 2011
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Revision History:
;	Who		Date	Description
;	sgt		TBD		Initial Release
;==========================================================================

#include "jobeaux.inc"
	SUBTITLE "EEPROM Module"

	GLOBAL	UpdateEEPROM

	EXTERN	DisplayF

EEPROM	CODE

;==========================================================================
; Update the EEPROM data.
; Modifies: bank
;==========================================================================
UpdateEEPROM:
		; Offset 0: Version
		banksel	EEADRL				; Bank 3
		movlw	EE_VERSION
		movwf	EEADRL				; Set EEPROM address
		movlw	EE_CURRENT_VERSION
		movwf	EEDATL				; And the data
		call	eewrite

		; Offset 1: Fahrenheit/Celsius display
		banksel	EEADRL				; Bank 3
		movlw	EE_FC_CONFIG
		movwf	EEADRL				; Set EEPROM address
		movfw	DisplayF
		movwf	EEDATL				; And the data
		call	eewrite
		return

;==========================================================================
; Write the current data in EEDATL to the address in EEADRL.
; Modifies: bank
;==========================================================================
eewrite:
		banksel	EECON1				; Bank 3
		bcf		EECON1, CFGS
		bcf		EECON1, EEPGD

		bsf		EECON1, WREN		; Enable writes
		bcf		INTCON, GIE			; Disable interrupts
		movlw	0x55				; output special sequence to show
		movwf	EECON2				; we really do want to write it
		movlw	0xAA
		movwf	EECON2
		bsf		EECON1, WR			; Go for it!
		bsf		INTCON, GIE			; Enable interrupts
		bcf		EECON1, WREN		; Disable writes

		; Wait for completion
wwait:	btfsc	EECON1, WR
		goto	wwait
		return

	end
