;==========================================================================
;  Jobeaux Processor Global definitions
; 
;  Copyright (C) 2011
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Revision History:
;	Who		Date	Description
;	sgt		TBD		Initial Release
;==========================================================================
#include "p16f1827.inc"
	TITLE "Jobeaux Processor, rev 0.0"
	RADIX DEC

;==========================================================================
; Define I/O ports
;==========================================================================
TIMER_RESET_IN		EQU	RA5			; Reset switch
TIMER_RESET_PORT	EQU	PORTA

LCD_ENABLE_OUT		EQU	RA7			; LCD Display /CSB
LCD_ENABLE_PORT		EQU PORTA

LCD_MODE_OUT		EQU	RA1			; LCD Display RS
LCD_MODE_PORT		EQU	PORTA

MOTOR_ROT_IN		EQU	RB0			; Motor rotation sensor (interrupt line)
MOTOR_ROT_PORT		EQU	PORTB

MOTOR_SPEED_IN		EQU RA4			; Hi/Lo speed switch
MOTOR_SPEED_PORT	EQU PORTA

MOTOR_DIR_OUT		EQU	RA2			; Motor direction (Phase)
MOTOR_DIR_PORT		EQU	PORTA

MOTOR_BRAKE_OUT		EQU	RA6			; Brake (Mode) control
MOTOR_BRAKE_PORT	EQU	PORTA

MOTOR_CTL_OUT		EQU	RB3			; DRV8801 Enable (PWM)
MOTOR_CTL_PORT		EQU	PORTB
MOTOR_CTL_TRIS		EQU	TRISB
PWM_DUTY_SLOW		EQU	4			; CCPR1L value for half-speed
PWM_DUTY_FAST		EQU	8			;  full speed
PWM_PERIOD			EQU	7

HEATER_CTL_OUT		EQU	RB5			; Heater on/off
HEATER_CTL_PORT		EQU	PORTB

;==========================================================================
; Starting value for Timer1.
; This causes the timer to overflow (and interrupt) at a rate of
; TICKS_PER_SEC
;==========================================================================
;T1_LP_START_HI		EQU	0xC0		; Using LP clock (32.768 KHz)
;T1_LP_START_LO		EQU	0x00		; Values for 2 ticks/sec
T1_LP_START_HI		EQU	0xE0		; Using LP clock (32.768 KHz)
T1_LP_START_LO		EQU	0x00		; Values for 4 ticks/sec

;T1_IC_START_HI		EQU	0x0B		; Using instruction clock/8 (125KHz)
;T1_IC_START_LO		EQU	0xDC		; Values for 2 ticks/sec
T1_IC_START_HI		EQU	0x85		; Using instruction clock/8 (125KHz)
T1_IC_START_LO		EQU	0xEE		; Values for 4 ticks/sec

TICKS_PER_SEC		EQU	4

;==========================================================================
; TOO_COLD_COUNT is the value that tells us whether to turn the heater on
; or not.  If the current temperature is more than TOO_COLD_COUNT ticks
; below the desired temperature, we turn on.
;==========================================================================
TOO_COLD_COUNT_LO	EQU	3
TOO_COLD_COUNT_HI	EQU	0

STABLE_COUNT_LO		EQU	3			; Should be <= TOO_COLD_COUNT
STABLE_COUNT_HI		EQU	0

OFF_COUNT_OFFSET_LO	EQU	1
OFF_COUNT_OFFSET_HI	EQU	0

;==========================================================================
; Motor states
;==========================================================================
MSTATE_STOPPED		EQU	0			; Motor is stopped
MSTATE_STARTING		EQU	1			; Motor is starting
MSTATE_TURNING		EQU	2			; Motor is running
MSTATE_STOPPING		EQU	3			; Motor is coasting to a stop

; Phase bits (these are bit numbers)
PHASE_RISING		EQU	0
PHASE_FALLING		EQU	1

; How many motor revolutions before we reverse direction
REVERSE_COUNT		EQU	2

; How long (ticks) to stay in the STOPPING & STOPPED states
COAST_TIME			EQU	3
STOP_TIME			EQU	3

;==========================================================================
; Heater states
;==========================================================================
HSTATE_IDLE			EQU	0			; Heater OFF; turn on if temp drops
									; TOO_COLD ticks below desired temp
HSTATE_HEATING		EQU	1			; Heater ON; turn off when temp reaches
									; computed value below desired temp
HSTATE_PAUSE		EQU	2			; Heater OFF; just pausing for a second
									; or so before going to STABILIZE state
HSTATE_STABILIZE	EQU	3			; Heater OFF; cannot go back on until
									; peak drops by STABLE_COUNT.  Water is
									; still heating due to retained heat in
									; heater element

; How long (ticks) to stay in the PAUSE state
PAUSE_TIME			EQU	40


;==========================================================================
; EEPROM configuration bytes
; Only one byte is currently defined.
; If you define any more, you may have to modify the code in
; ConfigureUnits (temp.asm) to wait for write completion
;==========================================================================
; OFFSETS
EE_VERSION			EQU	0			; Version number
EE_FC_CONFIG		EQU	1			; Fahrenheit/Celsius display

; Define current version number.  This value gets stored at offset
; EE_VERSION in the EEPROM
EE_CURRENT_VERSION	EQU	0

;==========================================================================
; Non-ASCII display characters
;==========================================================================
DEGREES				EQU	0xF2		; Degree symbol for display
INTERNAL_CLOCK_IND	EQU 0xFD		; Not-equal sign
