;==========================================================================
;  Jobeaux Processor Temperature Control Module
; 
;  Copyright (C) 2011
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Revision History:
;	Who		Date	Description
;	sgt		TBD		Initial Release
;==========================================================================

#include "jobeaux.inc"
	SUBTITLE "Temp Control Module"

	EXTERN	msDelay, DisplaySetupScreen			; In display.asm
	EXTERN	OKUpdate							; In timer.asm
	EXTERN	UpdateEEPROM						; In eeprom.asm
	EXTERN	Rotate, MotorState, Phase			; DEBUG

	GLOBAL	InitHeater, InitTempDisplay, ConfigureUnits
	GLOBAL	ReadTemp, TendHeater, CheckWaterLevel
	GLOBAL	DisplaySetTemp, DisplayCurTemp, DisplayLowTemp
	GLOBAL	CountToTemp
	GLOBAL	SetupScr1, SetupScr2, SetupScr3
	GLOBAL	DisplayF

		UDATA_SHR
DisplayF		RES	1			; 1: display degrees F, 0: display C
DesiredHi		RES	1			; Desired temp from R6
DesiredLo		RES	1
CurrentHi		RES	1			; Currently read temp from thermistor
CurrentLo		RES	1

		UDATA
; Working registers for temp conversion
TempHi			RES	1			; The temp we're looking for
TempLo			RES	1
EntryHi			RES	1			; Current table entry value
EntryLo			RES	1
PrevHi			RES	1			; Previous table entry value
PrevLo			RES	1
OffCountHi		RES	1			; When to turn off heater
OffCountLo		RES	1
RecentPeakHi	RES	1			; Recent peak temp when stabilizing
RecentPeakLo	RES	1
SmoothedHi		RES	1			; Smoothed current temp
SmoothedLo		RES	1
TempHTBCD		RES	1			; Temp Hundreds/Tens digit (BCD)
TempOFBCD		RES	1			; Temp Ones/Tenths digit (BCD)
HeaterState		RES	1			; Current heater state
PauseCount		RES	1

tmp				RES	1			; Temporary working register
equal			RES	1
innerCount		RES	1			; Display Configuration inner loop counter
outerCount		RES	1			; Display Configuration outer loop counter

TEMP	CODE

;==========================================================================
; ADC values for temperatures from 50-110 degrees F
; This saves the wimpy PIC from having to do a lot of math.
; These are based on the Steinhart-Hart Equation for
; "Material Z" (a=1.12924E-3, b=2.34108E-4, c=0.87755E-7)
; Computed at 1/2 degree intervals, interpolate to get finer
; intervals.
;==========================================================================
FTemps:
MinSel:	; Minimum valid desired temp
		dw   12,  26,  39,  53,  66,  79,  93, 106		;  50.0 -  53.5
		dw  119, 132, 144, 157, 170, 182, 195, 207		;  54.0 -  57.5
		dw  219, 231, 243, 255, 267, 278, 290, 301		;  58.0 -  61.5
		dw  313, 324, 335, 346, 357, 368, 379, 389		;  62.0 -  65.5
		dw  400, 410, 421, 431, 441, 451, 461, 471		;  66.0 -  69.5
		dw  481, 491, 500, 510, 519, 529, 538, 547		;  70.0 -  73.5
		dw  556, 565, 574, 583, 592, 601, 609, 618		;  74.0 -  77.5
		dw  626, 634, 643, 651, 659, 667, 675, 683		;  78.0 -  81.5
		dw  690, 698, 706, 713, 721, 728, 735, 743		;  82.0 -  85.5
		dw  750, 757, 764, 771, 778, 785, 791, 798		;  86.0 -  89.5
		dw  805, 811, 818, 824, 830, 837, 843, 849		;  90.0 -  93.5
		dw  855, 861, 867, 873, 879, 884, 890, 896		;  94.0 -  97.5
		dw  901, 907, 912, 918, 923, 928, 934, 939		;  98.0 - 101.5
		dw  944, 949, 954, 959, 964, 969, 974, 978		; 102.0 - 105.5
		dw  983, 988, 992, 997,1001,1006,1010,1015		; 106.0 - 109.5
		dw 1019,1023									; 110.0 - 110.5



;==========================================================================
; ADC values for temperatures from 10-44 degrees C
; We can't actually get a reading of 44.0, as the ADC is only 10 bits.
; Highest setting is actually 43.6
; Essentially same range as above table, but for Celsius readout
;==========================================================================
CTemps:
		dw   12,  37,  61,  85, 108, 132, 155, 177		; 10.0 - 13.5
		dw  199, 221, 243, 264, 285, 306, 326, 346		; 14.0 - 17.5
		dw  366, 385, 404, 423, 441, 459, 477, 495		; 18.0 - 21.5
		dw  512, 529, 545, 562, 578, 594, 609, 624		; 22.0 - 25.5
		dw  639, 654, 668, 683, 697, 710, 724, 737		; 26.0 - 29.5
		dw  750, 763, 775, 787, 799, 811, 823, 834		; 30.0 - 33.5
		dw  845, 856, 867, 877, 888, 898, 908, 918		; 34.0 - 37.5
		dw  927, 937, 946, 955, 964, 973, 981, 990		; 38.0 - 41.5
		dw  998,1006,1014,1022,1029						; 42.0 - 44.0


;==========================================================================
; Initialize the heater
; Nothing much to do.  Only control is Heater Ctl.  It's an inverted
; signal, so set to 1 to turn the heater off for now.
; Also initializes the Heater state to IDLE
; Modifies: bank
;==========================================================================
InitHeater:
		banksel	HeaterState
		movlw	HSTATE_IDLE
		movwf	HeaterState

		banksel	HEATER_CTL_PORT		; Bank 0
		bsf		HEATER_CTL_PORT, HEATER_CTL_OUT

		return


;==========================================================================
; Initialize the temperature display
; Read the configuration byte in EEPROM to determine if we are displaying
; in Fahrenheit or Celsius
; Modifies: bank
;==========================================================================
InitTempDisplay:
		clrf	DisplayF			; Assume Celsius

		; Read the Display configuration from EEPROM
		banksel	EEADRL				; Bank 3
		movlw	EE_FC_CONFIG
		movwf	EEADRL
		bcf		EECON1, CFGS
		bcf		EECON1, EEPGD
		bsf		EECON1, RD

		movfw	EEDATL
		btfss	STATUS, Z			; zero = C, non-zero = F
		incf	DisplayF, F			; Change to Fahrenheit
		return


;==========================================================================
; Read the desired and current water temp values & saves them
; Modifies: bank
;==========================================================================
ReadTemp:
		banksel	ADCON0				; Bank 1
		movlw	B'00101101'			; Select AN11 (Desired Temp)
		movwf	ADCON0				; Turn ADC on
		call	sampleTime			; Acquisition delay
		bsf		ADCON0, ADGO		; Start Conversion
		btfsc	ADCON0, ADGO		; Wait for completion
		goto	$-1
		movfw	ADRESH				; Get the high 2 bits
		movwf	DesiredHi
		movfw	ADRESL				; And the lower 8 bits
		movwf	DesiredLo

		movlw	B'00000001'			; Select AN0 (Current Temp)
		movwf	ADCON0				; Turn ADC on
		call	sampleTime			; Acquisition delay
		bsf		ADCON0, ADGO		; Start Conversion
		btfsc	ADCON0, ADGO		; Wait for completion
		goto	$-1
		movfw	ADRESH				; Get the high 2 bits
		movwf	CurrentHi
		movfw	ADRESL				; And the lower 8 bits
		movwf	CurrentLo

		return

;==========================================================================
; Acquisition delay function.
; Wastes a few microseconds
;==========================================================================
sampleTime:
		goto $+1
		goto $+1
		goto $+1
		return


;==========================================================================
; Verify that the water level is not too low.
; Returns value in W
; 0 = Too Low
; 1 = OK
; Modifies: bank
;==========================================================================
CheckWaterLevel:
		; Compare temp select value vs MinSel value
		; If below MinSel we assume float switch is turned on
		banksel	EEADRL				; Bank 3
		movlw	LOW MinSel			; Set MinSel addr
		movwf	EEADRL
		movlw	HIGH MinSel
		movwf	EEADRH
		bcf		EECON1, CFGS
		bsf		EECON1, EEPGD
		bcf		INTCON, GIE			; Disable interrupts
		bsf		EECON1, RD			; Initiate read of MinSel to EEDAT regs
		nop
		nop
		bsf		INTCON, GIE			; Reenable interrupts

		movfw	EEDATL				; MinSel in W
		subwf	DesiredLo, W		; Desired in f
		movfw	EEDATH
		subwfb	DesiredHi, W
		btfss	STATUS, C			; Carry set if W <= f
		retlw	0					; Desired < MinSel
		retlw	1					; Desired >= MinSel


;==========================================================================
; Turn the heater on if the current temp is below the requested temp.
; May have to get clever about when to turn it off, as the heater element
; will have some retained heat at turn off.  Will have to experiment to
; see how much this affects the temperature.
; Modifies: bank, page
;==========================================================================
TendHeater:
		; If water level is too low, turn off heater & bail
		pagesel	CheckWaterLevel
		call	CheckWaterLevel
		andlw	0xFF
		btfsc	STATUS, Z
		goto	waterFault

		banksel	HeaterState
		movfw	HeaterState			; Dispatch to appropriate state handler
		brw
		goto	IdleState			; HSTATE_IDLE
		goto	HeatingState		; HSTATE_HEATING
		goto	PauseState			; HSTATE_PAUSE
		goto	StabilizeState		; HSTATE_STABILIZE

IdleState:
		; If too cold, turn the heater ON
		; Too cold if (Current + TOO_COLD_COUNT) < Desired
		; Use PrevLo/PrevHi to hold intermediate result
		banksel	PrevLo
		movfw	CurrentLo
		addlw	TOO_COLD_COUNT_LO
		movwf	PrevLo
		movfw	CurrentHi
		movwf	PrevHi
		movlw	TOO_COLD_COUNT_HI
		addwfc	PrevHi, F

		; Now compare Desired with Prev
		movfw	DesiredLo			; Desired in W
		subwf	PrevLo, W			; Prev in f
		movfw	DesiredHi
		subwfb	PrevHi, W
		btfsc	STATUS, C			; Carry set if W <= f
		return						; Desired <= (Current + TOO_COLD_COUNT)

		; Desired > (Current + TOO_COLD_COUNT): Compute off value
		; Off count = Desired + (Current - Desired)/4 + OFF_COUNT_OFFSET
		; Since Desired > Current, the second term is <= 0
		banksel	OffCountLo
		movfw	DesiredLo			; OffCount = Current - Desired
		subwf	CurrentLo, W
		movwf	OffCountLo
		movfw	DesiredHi
		subwfb	CurrentHi, W
		movwf	OffCountHi
		asrf	OffCountHi, F		; Divide by 2
		rrf		OffCountLo, F
		asrf	OffCountHi, F		; And again to get 4
		rrf		OffCountLo, F
		movfw	DesiredLo			; Now add Desired to the result
		addwf	OffCountLo, F
		movfw	DesiredHi
		addwfc	OffCountHi, F
		movlw	OFF_COUNT_OFFSET_LO
		addwf	OffCountLo, F
		movlw	OFF_COUNT_OFFSET_HI
		addwfc	OffCountHi, F

		; Now set new state & turn the heater on
		banksel	HeaterState
		movlw	HSTATE_HEATING
		movwf	HeaterState			; Set new state

		banksel	HEATER_CTL_PORT		; Bank 0
		bcf		HEATER_CTL_PORT, HEATER_CTL_OUT
		return

HeatingState:
		; User can adjust desired temp during heating phase.
		; If a temp lower than the current temp is selected, we need
		; to turn off the heater and bail (go back to IDLE state)
		; right now.
		movfw	DesiredLo			; Desired in W
		subwf	CurrentLo, W		; Current in f
		movfw	DesiredHi
		subwfb	CurrentHi, W
		btfsc	STATUS, C			; Carry set if W <= f
		goto	InitHeater			; It is - back to idle state

		; If desired < off count, also bail.  User has adjusted desired
		; temp down, but still above curent temp.
		; (Does this make the above test redundant?)
		movfw	OffCountLo			; OffCount in W
		subwf	DesiredLo, W		; Desired in f
		movfw	OffCountHi
		subwfb	DesiredHi, W
		btfss	STATUS, C			; Carry set if W <= f
		goto	InitHeater			; It isn't - back to idle state

		; If Current >= previously computed off count, turn off heater
		; Off Count should always be <= Desired - if they're equal, we
		; will have bailed out above.  Otherwise we go into PAUSE
		; state when we reach the count.
		banksel	OffCountLo
		movfw	OffCountLo			; Computed off count in W
		subwf	CurrentLo, W		; Current in f
		movfw	OffCountHi
		subwfb	CurrentHi, W
		btfss	STATUS, C			; Carry set if W <= f
		return						; Current < off count - keep heating

		; Current >= Desired: Set new state
		banksel	HeaterState
		movlw	HSTATE_PAUSE
		movwf	HeaterState
		movlw	PAUSE_TIME
		movwf	PauseCount

		; Initialize Recent Peak value
		banksel	RecentPeakLo
		movfw	CurrentLo
		movwf	RecentPeakLo
		movwf	SmoothedLo
		movfw	CurrentHi
		movwf	RecentPeakHi
		movwf	SmoothedHi

		; And fall through to heater off code

waterFault:
		; Turn off heater & go back to IDLE state.
		; We'll start over from scratch when the fault is cleared.
		goto InitHeater

PauseState:
		; Wait for PAUSE_COUNT clock ticks
		call	Smooth
		movf	OKUpdate, F
		btfsc	STATUS, Z
		return

		banksel	PauseCount
		decfsz	PauseCount, F
		return

		banksel	HeaterState
		movlw	HSTATE_STABILIZE
		movwf	HeaterState
		return
		
StabilizeState:
		; User can adjust desired temp during heating phase.
		; If a temp lower than the current temp is selected, we need
		; to turn off the heater and bail (go back to IDLE state)
		; right now.  Also causes exit of STABILIZE state on overshoot
		; of desired temp.
		movfw	DesiredLo			; Desired in W
		subwf	CurrentLo, W		; Current in f
		movfw	DesiredHi
		subwfb	CurrentHi, W
		btfsc	STATUS, C			; Carry set if W <= f
		goto	InitHeater			; It is - back to idle state

		call	Smooth

		; We need to (try to) remain in the STABILIZE state as long as
		; the temperature keeps increasing.
		banksel	RecentPeakLo
		movfw	RecentPeakLo		; Recent peak count in W
		subwf	SmoothedLo, W		; Current in f
		movfw	RecentPeakHi
		subwfb	SmoothedHi, W
		btfss	STATUS, C			; Carry set if W <= f
		goto	checkStable			; Current < Recent Peak

		; Current >= Recent Peak: update peak value
		movfw	SmoothedLo
		movwf	RecentPeakLo
		movfw	SmoothedHi
		movwf	RecentPeakHi
		return

checkStable:
		; Only check once a second (twice a second if using internal clock)
		; FixFixFix - This is now 1/4 second
		movf	OKUpdate, F
		btfsc	STATUS, Z
		return

		; See if we've dropped enough to consider the temperature
		; stabilized.  Using PrevLo/Hi as temp again
		banksel	PrevLo
		movlw	STABLE_COUNT_LO
		addwf	SmoothedLo, W
		movwf	PrevLo
		movlw	STABLE_COUNT_HI
		addwfc	SmoothedHi, W
		movwf	PrevHi

		movfw	RecentPeakLo		; Recent peak count in W
		subwf	PrevLo, W			; (Current + STABLE_COUNT) in f
		movfw	RecentPeakHi
		subwfb	PrevHi, W
		btfsc	STATUS, C			; Carry set if W <= f
		return						; No state change

		banksel	HeaterState
		movlw	HSTATE_IDLE			; (Current + STABLE_COUNT) < Recent Peak
		movwf	HeaterState			; Save new state
		return


;==========================================================================
; Smooth the temp readings a bit
;==========================================================================
Smooth:
		banksel	SmoothedLo
		movfw	SmoothedLo
		addwf	SmoothedLo, W		; old * 2
		btfsc	STATUS, C
		incfsz	SmoothedHi, F
		nop
		addwf	SmoothedLo, F		; *3
		btfsc	STATUS, C
		incfsz	SmoothedHi, F
		nop
		movfw	SmoothedHi
		addwf	SmoothedHi, W
		addwf	SmoothedHi, F
		movfw	CurrentLo
		addwf	SmoothedLo, F
		movfw	CurrentHi			; + Current
		addwfc	SmoothedHi, F
		asrf	SmoothedHi, F		; / 4
		rrf		SmoothedLo, F
		asrf	SmoothedHi, F
		rrf		SmoothedLo, F
		return


;==========================================================================
; Convert the ADC value to temperature for display
; On entry, W specifies which temp to convert:
;	0 = Desired
;	1 = Current
;
; On return, W indicates success/failure
;	0 = Temp < 50 degrees
;	1 = Success
; Modifies: bank
;==========================================================================
CountToTemp:
		banksel	TempHi
		; Copy selected temp to working regs
		andlw	0xFF
		btfsc	STATUS, Z
		goto	copySel

		; Current temp selected - copy it
		movfw	CurrentHi
		movwf	TempHi
		movfw	CurrentLo
		movwf	TempLo
		goto	convert

copySel:
		; Desired temp selected - copy it
		movfw	DesiredHi
		movwf	TempHi
		movfw	DesiredLo
		movwf	TempLo

convert:
		; Initialize temp digits to 50.0 if F, 10.0 if C
		movlw	0x01				; Assume C
		btfsc	DisplayF, 0			; Is it?
		movlw	0x05				; No, it's F
		movwf	TempHTBCD
		movlw	0x00
		movwf	TempOFBCD

		; Is the temp less than the lowest value we know how to convert?
		banksel	EEADRL				; Bank 3
		movlw	LOW CTemps			; Set Temps[0] addr - C or F
		btfsc	DisplayF, 0
		movlw	LOW FTemps
		movwf	EEADRL
		movlw	HIGH CTemps
		btfsc	DisplayF, 0
		movlw	HIGH FTemps
		movwf	EEADRH
		bcf		EECON1, CFGS
		bsf		EECON1, EEPGD
		bcf		INTCON, GIE			; Disable interrupts
		bsf		EECON1, RD			; Initiate read of Temps[0] to EEDAT regs
		nop
		nop
		bsf		INTCON, GIE			; Reenable interrupts

		movfw	EEDATL				; Temps[0] in W
		banksel	EntryLo
		movwf	EntryLo
		subwf	TempLo, W			; Temp in f
		banksel	EEDATH				; Bank 3
		movfw	EEDATH
		banksel	EntryHi
		movwf	EntryHi
		subwfb	TempHi, W
		btfss	STATUS, C			; Carry set if W <= f
		retlw	0					; f < W: Can't convert

		; Scan Temps table to find bounding .5 degree entries
scan:
		banksel	EntryLo
		movfw	EntryLo				; Save previous values in case
		movwf	PrevLo				; we need to interpolate
		movfw	EntryHi
		movwf	PrevHi

		banksel	EEADRL				; Bank 3
		incfsz	EEADRL, F			; Move to next table entry
		goto	readit
		incfsz	EEADRH, F			; page wrap
		nop

readit:
		bcf		EECON1, CFGS
		bsf		EECON1, EEPGD
		bcf		INTCON, GIE			; Disable interrupts
		bsf		EECON1, RD			; Initiate read of Temps[0] to EEDAT regs
		nop
		nop
		bsf		INTCON, GIE			; Reenable interrupts

		; Compare table entry to temp we're converting
		; If temp is >= table entry, we've found the right entry
		movfw	EEDATL
		banksel	EntryLo
		movwf	EntryLo
		movlw	1
		movwf	equal				; Assume equal
		movfw	TempLo				; Temp in W
		subwf	EntryLo, W			; Temps[n] in f
		btfss	STATUS, Z
		clrf	equal				; Low bytes not equal
		banksel	EEDATH				; Bank 3
		movfw	EEDATH
		banksel	EntryHi
		movwf	EntryHi
		movfw	TempHi
		subwfb	EntryHi, W
		btfsc	STATUS, C			; Carry set if W <= f
		goto	found

		; Haven't found it yet - update BCD by .5 & try again
		movlw	5
		call	UpdateBCD
		goto	scan

found:
		banksel	equal
		btfss	STATUS, Z
		clrf	equal				; High bytes not equal
		movf	equal, F
		btfsc	STATUS, Z			; Is it right on?
		goto	interpolate			; Nope
		movlw	5					; Yes - just update BCD by .5 & we're done
		call	UpdateBCD
		retlw	1

		; Interpolate to nearest .1 degree
interpolate:
		; Compute diff between entry & previous
		; Value of result will be between 4 and 14
		banksel	PrevLo
		movfw	PrevLo
		subwf	EntryLo, F			; Leave result in EntryLo

		; Compute diff between temp & prev
		subwf	TempLo, F			; Leave result in TempLo

		; Times 5 - biggest diff between entries is 14
		; so guaranteed to fit in a byte
		movfw	TempLo
		lslf	WREG, W				; * 2
		lslf	WREG, W				; * 4
		addwf	TempLo, F			; * 5 result in TempLo

		; Subtract half of EntryLo - this provides proper rounding
		movfw	EntryLo
		asrf	WREG, W
		subwf	TempLo, F
		btfss	STATUS, C			; Are we done?

		retlw	1					; Yes - close enough already	

		; Divide by successive subtraction
		clrf	tmp
		movfw	EntryLo
subdiv:
		subwf	TempLo, F			; Carry is 0 if TempLo was < EntryLo
		incf	tmp, F
		btfss	STATUS, C
		goto	divdone				; Which means we're done...
		goto	subdiv				; and loop

divdone:
		; Final result should be in range 1-5
		movfw	tmp
		call	UpdateBCD
		retlw	1



;==========================================================================
; Update the BCD temperature value
; Number of tenths to add is in W (1-5)
; Modifies: bank
;==========================================================================
UpdateBCD:
		banksel	TempOFBCD
		addwf	TempOFBCD, F		; Add the specified number of tenths
		movfw	TempOFBCD			; Now need to see if we went over
		andlw	0x0F
		sublw	9					; Sets carry if W <= 9
		btfsc	STATUS, C
		return						; Good to go.

		movlw	0x06				; Get tenths back in range & inc ones
		addwf	TempOFBCD, F
		swapf	TempOFBCD, W		; Now see if that wrapped
		andlw	0x0F
		sublw	9					; Can't go past 9
		btfsc	STATUS, C
		return						; Didn't

		movlw	0x60
		addwf	TempOFBCD, F		; Force upper nibble (ones) to zero
		incfsz	TempHTBCD, F		; Bump tens (can't skip)
		movfw	TempHTBCD			; Now see if it's wrapped
		andlw	0x0F
		sublw	9
		btfsc	STATUS, C
		return						; No - we're good

		movlw	0x06				; Sure did - bump it to 0 and inc hundreds
		addwf	TempHTBCD, F		; Don't worry about hundreds wrapping
		return

;==========================================================================
; Display the requested temperature
; Column is in W on entry
;==========================================================================
DisplaySetTemp:
		brw
		retlw	'S'					; Column 0
		retlw	'e'					; Column 1
		retlw	't'					; Column 2
		retlw	':'					; Column 3
		retlw	' '					; Column 4
		retlw	' '					; Column 5
		goto	tempH				; Column 6: Hundreds digit
		goto	tempT				; Column 7: Tens digit
		goto	tempO				; Column 8: Ones digit
		retlw	'.'					; Column 9
		goto	tempF				; Column 10: Tenths digit
		retlw	DEGREES				; Column 11
		goto	CorF				; Column 12
		retlw	' '					; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15
; DEBUG
		goto	hi7
		goto	hi6
		goto	hi5
		goto	hi4
		goto	hi3
		goto	hi2
		goto	hi1
		goto	hi0
		goto	lo7
		goto	lo6
		goto	lo5
		goto	lo4
		goto	lo3
		goto	lo2
		goto	lo1
		goto	lo0

hi7:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x80
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

hi6:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x40
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

hi5:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x20
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

hi4:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x10
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

hi3:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x08
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

hi2:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x04
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

hi1:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x02
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

hi0:
;		banksel	OffCountHi
;		movfw	OffCountHi
		movfw	DesiredHi
		andlw	0x01
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo7:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x80
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo6:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x40
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo5:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x20
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo4:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x10
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo3:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x08
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo2:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x04
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo1:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x02
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

lo0:
;		banksel	OffCountLo
;		movfw	OffCountLo
		movfw	DesiredLo
		andlw	0x01
		btfss	STATUS, Z
		retlw	'1'
		retlw	'0'

;==========================================================================
; Display the current temperature
; Column is in W on entry
;==========================================================================
DisplayCurTemp:
		brw
		retlw	'T'					; Column 0
		retlw	'e'					; Column 1
		retlw	'm'					; Column 2
		retlw	'p'					; Column 3
		retlw	':'					; Column 4
		retlw	' '					; Column 5
		goto	tempH				; Column 6: Hundreds digit
		goto	tempT				; Column 7: Tens digit
		goto	tempO				; Column 8: Ones digit
		retlw	'.'					; Column 9
		goto	tempF				; Column 10: Tenths digit
		retlw	DEGREES				; Column 11
		goto	CorF				; Column 12
;		retlw	' '					; Column 13
		goto	DisplayPhase		; DEBUG
;		retlw	' '					; Column 14
		goto	DisplayHState		; DEBUG
;		retlw	' '					; Column 15
		goto	DisplayMState		; DEBUG


DisplayPhase:						; DEBUG
		movfw	Phase				; DEBUG
		addlw	'0'					; DEBUG
		return						; DEBUG

DisplayHState:						; DEBUG
;		banksel	HeaterState			; DEBUG
;		movfw	HeaterState			; DEBUG
		movfw	Rotate				; DEBUG
		addlw	'0'					; DEBUG
		return						; DEBUG

DisplayMState:						; DEBUG
		banksel	MotorState			; DEBUG
		movfw	MotorState			; DEBUG
		addlw	'0'					; DEBUG
		return						; DEBUG

;==========================================================================
; Return the hundreds digit of the converted temp as ASCII
; Leading zero is suppressed
; Modifies: bank
;==========================================================================
tempH:
		banksel	TempHTBCD
		movfw	TempHTBCD
		swapf	WREG, W
		andlw	0x0F
		btfsc	STATUS, Z
		retlw	' '					; It's zero - return a space
convertToASC:
		addlw	'0'					; Convert to ASCII digit
		return

;==========================================================================
; Return the tens digit of the converted temp as ASCII
; Modifies: bank
;==========================================================================
tempT:
		banksel	TempHTBCD
		movfw	TempHTBCD
		andlw	0x0F				; Low nibble
		goto	convertToASC

;==========================================================================
; Return the ones digit of the converted temp as ASCII
; Modifies: bank
;==========================================================================
tempO:
		banksel	TempOFBCD
		movfw	TempOFBCD
		swapf	WREG, W				; High nibble
		andlw	0x0F
		goto convertToASC

;==========================================================================
; Return the tenths digit of the converted temp as ASCII
; Modifies: bank
;==========================================================================
tempF:
		banksel	TempOFBCD
		movfw	TempOFBCD
		andlw	0x0F				; Low nibble
		goto convertToASC

;==========================================================================
; Return the temperature units (C or F) based on configuration
;==========================================================================
CorF:
		btfsc	DisplayF, 0
		retlw	'F'
		retlw	'C'


;==========================================================================
; Return the tens digit of the minimum temperature for C or F
; based on configuration
;==========================================================================
CFMin:
		btfsc	DisplayF, 0
		retlw	'5'					; F: min is 50 degrees
		retlw	'1'					; C: min is 10 degrees


;==========================================================================
; Display a message showing the temperature is too low
; Column is in W on entry
;==========================================================================
DisplayLowTemp:
		brw
		retlw	'T'					; Column 0
		retlw	'e'					; Column 1
		retlw	'm'					; Column 2
		retlw	'p'					; Column 3
		retlw	':'					; Column 4
		retlw	' '					; Column 5
		retlw	'<'					; Column 6
		goto	CFMin				; Column 7: Tens digit
		retlw	'0'					; Column 8: Ones digit
		retlw	'.'					; Column 9
		retlw	'0'					; Column 10: Tenths digit
		retlw	DEGREES				; Column 11
		goto	CorF				; Column 12
		retlw	' '					; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15


;==========================================================================
; Configure whether temperature display is in F or C
; Modifies: bank, page
;==========================================================================
ConfigureUnits:
		; Set outer loop count
		banksel	outerCount
		movlw	9					; Give user 9 seconds
		movwf	outerCount			; to make changes

cuOuterLoop:						; Repeats at 1 sec intervals
		; Set inner loop count
		banksel	innerCount
		movlw	100
		movwf	innerCount

cuInnerLoop:						; Repeats at 10 msec intervals
		pagesel	DisplaySetupScreen
		call	DisplaySetupScreen

		; Wait for button unpressed
		banksel	TIMER_RESET_PORT	; Bank 0
		btfss	TIMER_RESET_PORT, TIMER_RESET_IN
		goto	$-1					; Still pressed

		; delay 10 msec
		movlw	10
		pagesel	msDelay
		call	msDelay

		; Check if button pressed
		movlw	1					; Prepare for toggle
		banksel	TIMER_RESET_PORT	; Bank 0
		btfss	TIMER_RESET_PORT, TIMER_RESET_IN
		xorwf	DisplayF, F			; Yes => toggle DisplayF

		; Decrement inner loop counter
		banksel	innerCount
		decfsz	innerCount, F
		goto	cuInnerLoop			; Non-zero: repeat inner loop

		; zero: decrement outer loop counter
		decfsz	outerCount, F
		goto	cuOuterLoop

		; outer loop counter zero: save to EEPROM and return
		; To minimize code, we always write the value back to EEPROM
		; even if it hasn't changed.  Should not be a problem since
		; this setup mode is rarely invoked.
		pagesel	UpdateEEPROM
		call	UpdateEEPROM
		return


;==========================================================================
; SetupScr1: Line 1 of setup screen
; Column is in W on entry
;==========================================================================
SetupScr1:
		brw
		retlw	'C'					; Column 0
		retlw	'o'					; Column 1
		retlw	'n'					; Column 2
		retlw	'f'					; Column 3
		retlw	'i'					; Column 4
		retlw	'g'					; Column 5
		retlw	'u'					; Column 6
		retlw	'r'					; Column 7
		retlw	'e'					; Column 8
		retlw	' '					; Column 9
		retlw	'U'					; Column 10
		retlw	'n'					; Column 11
		retlw	'i'					; Column 12
		retlw	't'					; Column 13
		retlw	's'					; Column 14
		retlw	' '					; Column 15

;==========================================================================
; SetupScr2: Line 2 of setup screen
; Column is in W on entry
; Modifies: bank
;==========================================================================
SetupScr2:
		btfss	DisplayF, 0
		goto	scr2Celsius

		brw
		retlw	'F'					; Column 0
		retlw	'a'					; Column 1
		retlw	'h'					; Column 2
		retlw	'r'					; Column 3
		retlw	'e'					; Column 4
		retlw	'n'					; Column 5
		retlw	'h'					; Column 6
		retlw	'e'					; Column 7
		retlw	'i'					; Column 8
		retlw	't'					; Column 9
		retlw	' '					; Column 10
		retlw	'('					; Column 11
		retlw	DEGREES				; Column 12
		retlw	'F'					; Column 13
		retlw	')'					; Column 14
		retlw	' '					; Column 15

scr2Celsius:
		brw
		retlw	'C'					; Column 0
		retlw	'e'					; Column 1
		retlw	'l'					; Column 2
		retlw	's'					; Column 3
		retlw	'i'					; Column 4
		retlw	'u'					; Column 5
		retlw	's'					; Column 6
		retlw	' '					; Column 7
		retlw	'('					; Column 8
		retlw	DEGREES				; Column 9
		retlw	'C'					; Column 10
		retlw	')'					; Column 11
		retlw	' '					; Column 12
		retlw	' '					; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15

;==========================================================================
; SetupScr3: Line 3 of setup screen
; Column is in W on entry
;==========================================================================
SetupScr3:
		brw
		retlw	'R'					; Column 0
		retlw	'e'					; Column 1
		retlw	's'					; Column 2
		retlw	'e'					; Column 3
		retlw	't'					; Column 4
		retlw	' '					; Column 5
		retlw	'i'					; Column 6
		retlw	'n'					; Column 7
		retlw	':'					; Column 8
		retlw	' '					; Column 9
		goto	displayOLC			; Column 10
		retlw	' '					; Column 11
		retlw	' '					; Column 12
		retlw	' '					; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15

displayOLC:
		banksel	outerCount
		movfw	outerCount
		addlw	'0'					; Now you know why it starts at 9
		return

	end
